import java.util.*; // For Random number generator and Linked List
import java.awt.Color; // For colorizing

/**
 * Class that represents a general Agent.  
 * We are designing to offer more flexibility than
 * one might expect to use.
 * 
 * @author Mark Guzdial guzdial@cc.gatech.edu
 */
public class Agent extends Turtle
{
  
  /////////////// fields //////////////////////
  
  /** class constant for probability of NOT turning */
  protected static final double PROB_OF_STAY = 1/10;
  
  /** class constant for top speed (max num steps can move in a timestep) */
  protected static final int MAXSPEED = 40;

  /** random number generator */
  protected static Random randNumGen = new Random();

  /** My current speed */
  protected int speed;

  /** My simulation */
  protected Simulation simulation;
  
  ////////////////////////////// Constructors ////////////////////////
  
  /**
   * Constructor that takes the model display (the original
   * position will be randomly assigned)
   * @param modelDisplayer thing that displays the model
   * @param thisSim my simulation
   */
  public Agent (ModelDisplay modelDisplayer,Simulation thisSim)
  {
    super(randNumGen.nextInt(modelDisplayer.getWidth()),
          randNumGen.nextInt(modelDisplayer.getHeight()),
          modelDisplayer);
    init(thisSim);
  }
  
  /** Constructor that takes the x and y and a model
   * display to draw it on
   * @param x the starting x position
   * @param y the starting y position
   * @param modelDisplayer the thing that displays the model
   * @param thisSim my simulation
   */
  public Agent (int x, int y, ModelDisplay modelDisplayer, 
               Simulation thisSim) 
  {
    // let the parent constructor handle it
    super(x,y,modelDisplayer);
    init(thisSim);
  }
  
  ////////////////// methods ////////////////////////////////////////

  /** Accessors */
  public void setSpeed(int velocity) {speed = velocity;}
  public int getSpeed() { return speed; }
  public Simulation getSimulation() {return simulation;}
  
  // All of these methods can be over-ridden by the subclasses!
  
  /**
   * Method to initialize the new wolf object
   */
  public void init(Simulation thisSim)
  {
    // turn some random direction
    this.turn(randNumGen.nextInt(360));
    
    // set my simulation
    simulation = thisSim;
    
    // Set up the speed (by default)
    speed = MAXSPEED;
    
    // Add to list of all Actors
    thisSim.add(this);

  }
  
  /**
   * Method that handles when an Agent dies
   */
  public void die() 
  { 
    // Leave a mark on the world where I died...
    this.setBodyColor(Color.red); 
    
    // Remove me from the agents list
    simulation.remove(this);
    
    // ask the model display to remove this
    // Think of this as "ask the viewable world to remove this turtle"
    //getModelDisplay().remove(this);
  }

  /**
   * Method to get the closest agent within the passed distance
   * to this agent but NOT this agent. 
   * We'll search the input list of the kind
   * of objects to compare to.
   * @param distance the distance to look within
   * @param list the list of agents to look at
   * @return the closest agent in the given distance or null
   */
  public Agent getClosest(double distance,LinkedList list)
  {
    // Keep track of closest so-far
    Agent closest = null;
    double closestDistance = 999;
    double currDistance = 0;
    Agent current = null;
    
    
    // loop through the linked list looking for the closest Agent
    for (int i = 0; i<list.size(); i++)
    {
      current = (Agent) list.get(i);
      if (current != this) {
        currDistance = current.getDistance(this.getXPos(),this.getYPos());
        if (currDistance < distance)
        {
          if (closest == null || currDistance < closestDistance)
          {
            closest = current;
            closestDistance = currDistance;
          }
        }
      }// current != this
    }// for loop
    return closest;
  }
  
  /** 
   * Method to count the number of agents within the 
   * range that are on the given list
   * @param range the number of steps within which to consider agents
   * @param list of agents to consider
   * @return the number of agents within range.
   **/
  public int countInRange(double range, LinkedList list){
    // Keep track of count in range
    int count = 0;
    double currDistance;
    Agent current = null;
    
    
    // loop through the linked list looking for the closest Agent
    for (int i = 0; i<list.size(); i++)
    {
      current = (Agent) list.get(i);
      currDistance = current.getDistance(this.getXPos(),this.getYPos());
      if (currDistance < range)
      {
        count++;
      }
    }// for loop
    
    return count;
  }
  
  /**
   * Method to act during a time step 
   * Default is to pick a random direction and 
   * move some random amount up to top speed
   */
  public void act()
  {
    // Default action: wander aimlessly
      
    // if the randome number is > prob of NOT turning then turn
    if (randNumGen.nextFloat() > PROB_OF_STAY)
    {
      this.turn(randNumGen.nextInt(360));
    }
      
    // go forward some random amount 
    forward(randNumGen.nextInt(speed));
  } // end act()

  
}